#!/usr/bin/env perl
#-*- Mode: perl; tab-width: 2; indent-tabs-mode: nil; c-basic-offset: 2 -*-

# Functions for getting service descriptions.
#
# Copyright (C) 2002 Ximian, Inc.
#
# Authors: Carlos Garnacho Parro <garnacho@tuxerver.net>
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU Library General Public License as published
# by the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Library General Public License for more details.
#
# You should have received a copy of the GNU Library General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307, USA.


# This function extracts known names and descriptions from the description hash
sub xst_service_list_get_info
{
	my ($service) = @_;
	my (%service_description_list);
	my $desc;

	# This is the hash that describes all the known services
	%service_description_list = 
	(
    # please, keep this list sorted
		# service => [long_name, description]
    "apache"            =>	[ "web server", "" ],
    "anacron"           =>      [ "Run missed scheduled jobs at startup", "Run cron jobs that were left out due to downtime" ],
    "apmd"              =>      [ "Automatic power management daemon", "apmd is used for monitoring battery status and logging it via syslog(8). It can also be used for shutting down the machine when the battery is low." ],
    "atd"               =>      [ "Run command at given time", "Runs commands scheduled by the at command at the time specified when at was run, and runs batch commands when the load average is low enough." ],
    "chargen"           =>      [ "", "An xinetd internal service which generate characters. The xinetd internal service which continuously generates characters until the connection is dropped.  This is the tcp version. You must enable xinetd to use this service." ],
    "chargen-udp"       =>      [ "", "An xinetd internal service which generate characters. The xinetd internal service which continuously generates characters until the connection is dropped.  This is the udp version. You must enable xinetd to use this service." ],
		"cron"              =>	[ "daemon to execute scheduled commands", "" ],
    "crond"             =>      [ "Run scheduled jobs", "cron is a standard UNIX program that runs user-specified programs at periodic scheduled times. vixie cron adds a number of features to the basic UNIX cron, including better security and more powerful configuration options." ],
		"cupsys"            =>	[ "Common Unix Printing System", "" ],
    "daytime"           =>      [ "", "An internal xinetd service which gets the current system time then prints it out in a format like this: 'Wed Nov 13 22:30:27 EST 2002'.  This is the tcp version. You must enable xinetd to use this service." ],
    "daytime-udp"   =>      [ "", "An internal xinetd service which gets the current system time then prints it out in a format like this: 'Wed Nov 13 22:30:27 EST 2002'.  This is the udp version. You must enable xinetd to use this service." ],
    "echo"              =>      [ "", "An xinetd internal service which echo's characters back to clients.  This is the tcp version. You must enable xinetd to use this service." ],
    "echo-udp"          =>      [ "", "An xinetd internal service which echo's characters back to clients.  This is the udp version. You must enable xinetd to use this service." ],
    "fam"               =>  [ "file alteration monitor", "" ],
    "firstboot"         =>      [ "", "Firstboot is a druid style program that runs on the first time" ],
    "gdm"               =>	[ "GNOME Display Manager", "gdm provides the GNOME equivalent of a login: prompt." ],
    "gpm"                   =>      [ "console mouse support", "GPM adds mouse support to text-based Linux applications such the Midnight Commander. Is also allows mouse-based console cut-and-paste operations, and includes support for pop-up menus on the console." ],
    "httpd"             =>      [ "web server", "Apache is a World Wide Web server.  It is used to serve HTML files and CGI." ],
		"inetd"             =>	[ "internet superserver", "inetd allows running one daemon to invoke several others, reducing load on the system." ],
		"iptables"          =>	[ "administration tool for IPv4 packet filtering and NAT", "Iptables is used to set up, maintain, and inspect the tables of IP packet filter rules in the Linux kernel." ],
    "irda"              =>      [ "", "IrDA stack for Linux" ],
    "isdn"              =>      [ "", "start and stop ISDN services" ],
    "keytable"          =>      [ "", "This package loads the selected keyboard map as set in /etc/sysconfig/keyboard.  This can be selected using the kbdconfig utility.  You should leave this enabled for most machines." ],
		"klogd"             =>	[ "Kernel Log Daemon", "klogd is a system daemon which intercepts and logs Linux kernel messages." ],
    "kudzu"             =>      [ "", "This runs the hardware probe, and optionally configures changed hardware." ],
    "lircd"             =>      [ "", "LIRC is a package that allows you to decode and send" ],
    "lircmd"            =>      [ "", "LIRC is a package that allows you to decode and send" ],
    "lpd"               =>      [ "", "lpd is the print daemon required for lpr to work properly. It is basically a server that arbitrates print jobs to printer(s)." ],
    "named"             =>      [ "dns server", "named (BIND) is a Domain Name Server (DNS) that is used to resolve host names to IP addresses." ],
    "netfs"             =>      [ "", "Mounts and unmounts all Network File System (NFS), SMB (Lan Manager/Windows), and NCP (NetWare) mount points." ],
    "network"           =>      [ "", "Activates/Deactivates all network interfaces configured to start at boot time." ],
    "nfs"               =>      [ "", "NFS is a popular protocol for file sharing across TCP/IP networks. This service provides NFS server functionality, which is configured via the /etc/exports file." ],
    "nfslock"           =>      [ "", "NFS is a popular protocol for file sharing across TCP/IP networks. This service provides NFS file locking functionality."],
    "ntpd"              =>      [ "", "ntpd is the NTPv4 daemon." ],
    "portmap"           =>      [ "DARPA port to RPC program number mapper", "The portmapper manages RPC connections, which are used by protocols such as NFS and NIS. The portmap server must be running on machines which act as servers for protocols which make use of the RPC mechanism." ],
    "postfix"           =>  [ "Mail Transport Agent", "" ],
		"postgresql"        =>	[ "database server", "" ],
    "ppp"               =>  [ "Point to Point Protocol daemon", "The  Point-to-Point Protocol (PPP) provides a method for transmitting datagrams over serial point-to-point links." ],
    "random"            =>      [ "", "Saves and restores system entropy pool for higher quality random number generation." ],
    "rawdevices"        =>      [ "", "This scripts assignes raw devices to block devices (such as hard drive partitions). This is for the use of applications such as Oracle. You can set up the raw device to block device mapping by editing the file /etc/sysconfig/rawdevices." ],
    "rhnsd"             =>      [ "", "This is a daemon which handles the task of connecting periodically to the Red Hat Network servers to check for updates, notifications and perform system monitoring tasks according to the service level that this server is subscribed for." ],
    "rsync"             =>      [ "", "The rsync server is a good addition to am ftp server, as it allows crc checksumming etc. You must enable xinetd to use this service." ],
    "saslauthd"         =>      [ "", "saslauthd is a server process which handles plaintext authentication requests on behalf of the cyrus-sasl library."],
    "sendmail"          =>      [ "", "Sendmail is a Mail Transport Agent, which is the program that moves mail from one machine to another." ],
    "servers"           =>      [ "", "An internal xinetd service, listing active servers. You must enable xinetd to use this service." ],
    "services"          =>      [ "", "An internal xinetd service, listing active services. You must enable xinetd to use this service." ],
    "setserial"         =>  [ "get/set serial port information", "setserial is a program designed to set and/or report the configuration information associated with a serial port." ],
    "sgi_fam"           =>      [ "", "FAM is a file monitoring daemon. It can be used to get reports when files change. You must enable xinetd to use this service." ],
    "snmpd"             =>      [ "", "Simple Network Management Protocol (SNMP) Daemon"],
		"ssh"               =>	[ "OpenSSH SSH server", "ssh is a program for logging into a remote machine and for executing commands on a remote machine." ],
    "sshd"              =>      [ "OpenSSH SSH server", "OpenSSH server daemon" ],
		"sysklogd"          =>	[ "Linux system logging utilities", "Sysklogd provides two system utilities which provide support for system  logging  and  kernel  message  trapping." ],
    "syslog"            =>      [ "", "Syslog is the facility by which many daemons use to log messages to various system log files.  It is a good idea to always run syslog." ],
    "time"              =>      [ "", "An RFC 868 time server. This protocol provides a site-independent, machine readable date and time. The Time service sends back to the originating source the time in seconds since midnight on January first 1900.  This is the tcp version. You must enable xinetd to use this service." ],
    "time-udp"          =>      [ "", "An RFC 868 time server. This protocol provides a site-independent, machine readable date and time. The Time service sends back to the originating source the time in seconds since midnight on January first 1900.  This is the udp version. You must enable xinetd to use this service." ],
		"urandom"           =>	[ "kernel random number source devices", "The  random  number  generator  gathers environmental noise from device drivers and other sources into an entropy pool. From this entropy pool random numbers are created." ],
    "vmware"            =>  [ "Intel(TM) x86(TM)-based virtual machine", "The vmware command starts an Intel X86-based virtual machine. This script manages the services needed to run VMware software." ],
    "winbind"           =>      [ "", "Starts and stops the Samba winbind daemon" ],
    "wine"              =>      [ "", "Allow users to run Windows(tm) applications by just clicking on them (or typing ./file.exe)" ],
    "xfs"               =>	[ "X font server", "Xfs is the X Window System font server. It supplies fonts to X Window System display servers." ],
    "xinetd"            =>      [ "", "xinetd is a powerful replacement for inetd. xinetd has access control machanisms, extensive logging capabilities, the ability to make services available based on time, and can place limits on the number of servers that can be started, among other things." ]
	);

	$desc = $service_description_list{$service};
	return @$desc;	
}


# This function determines if a service is "forbidden" or not (if runlevel-admin must show it or not)
sub xst_service_list_service_is_forbidden
{
	my ($service) = @_;
	my (%service_forbidden_list);
	my ($ret);
	
	# This is the hash that contains all the "forbidden" services
	%service_forbidden_list =
	(
		# TODO: we ought to separate the services depending on the distro that uses them
	
		# These are the forbidden services I found in Red Hat
		"halt" => 1,
		"functions" => 1,
		"killall" => 1,
	
		# These are the forbidden services I found in Debian Woody	
		"single" => 1,
		"sendsigs" => 1,
		"reboot" => 1,
		"rcS" => 1,
		"modutils" => 1,
		"hostname.sh" => 1,
		#"halt" => 1 in debian too
		"devpts.sh" => 1,
		"console-screen.sh" => 1,
		"checkroot" => 1,
		"checkfs" => 1,
		"bootmisc.sh" => 1,
		"checkfs.sh" => 1
	);
	
	$ret = $service_forbidden_list{$service};
	return $ret;
}
	



1;
